@@ -13,23 +13,23 @@
     license reproduced at the end of this file.
 =============================================================================*/
 
-// Tests valid re-definitions of macros.
+// Tests the handling of comments and <backslash><newline>.
 
-/* Excerpts from ISO C 6.8.3 "Examples".    */
-#define OBJ_LIKE        (1-1)
-#define FTN_LIKE(a)     ( a )
+// This test is currently disabled, because of the known problem in Wave, that
+// causes multiple __LINE__ macros on splitted lines after preprocessing
+// directives are expanded as the line number of the directive itself.
 
-// 19.1:
-//R #line 25 "t_5_022.cpp"
-#define OBJ_LIKE    /* white space */  (1-1) /* other */
-OBJ_LIKE              //R (1-1) 
+// Note: This unit test passes only if Wave was compiled with a defined
+//       BOOST_WAVE_PREPROCESS_ERROR_MESSAGE_BODY compile time constant 
+//       (which is the default).
+#define STR(x) #x
 
-// 19.2:
-//R #line 32 "t_5_022.cpp"
-#define FTN_LIKE( a     )(  /* note the white space */  \
-                        a  /* other stuff on this line
-                           */ )
-FTN_LIKE(3)           //R ( 3 ) 
+// 3.4: Comment and <backslash><newline> in #error line.
+//E t_5_004.cpp(29): fatal error: encountered #error directive or #pragma wave stop(): (29) Message of first physical line.         (30) Message of second physical and first logical line.  (32) Message of forth physical and third logical line.
+#error  (__LINE__) Message of first physical line. \
+        (__LINE__) Message of second physical and first logical line.  /*
+                   this comment splices the lines
+    */  (__LINE__) Message of forth physical and third logical line.
 
 /*-
  * Copyright (c) 1998, 2002-2005 Kiyoshi Matsui <kmatsui@t3.rim.or.jp>
