@@ -15,10 +15,12 @@
 
 // Tests translation limits.
 
-#undef NEST
-#define NEST 14
+#ifdef  X0F
 
-#include "t_5_035_15.hpp"
+#undef NEST
+#define NEST 0xf
+
+#endif
 
 /*-
  * Copyright (c) 1998, 2002-2005 Kiyoshi Matsui <kmatsui@t3.rim.or.jp>
