@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright (C) 2008-2009, International Business Machines Corporation and
+* Copyright (C) 2008, International Business Machines Corporation and
 * others. All Rights Reserved.
 *******************************************************************************
 *
@@ -13,7 +13,7 @@
 
 #if !UCONFIG_NO_FORMATTING
 
-//TODO: define it in compiler time
+//FIXME: define it in compiler time
 //#define DTITVINF_DEBUG 1
 
 
@@ -317,8 +317,7 @@ DateIntervalInfo::initializeData(const Locale& locale, UErrorCode& err)
     ures_close(calBundle);
     ures_close(rb);
     status = U_ZERO_ERROR;
-    locNameLen = uloc_getParent(parentLocale, parentLocale,
-                                ULOC_FULLNAME_CAPACITY,&status);
+    locNameLen = uloc_getParent(parentLocale, parentLocale,50,&status);
   } while ( locNameLen > 0 );
 }
 
@@ -575,12 +574,12 @@ U_CDECL_BEGIN
  * @param val2  the other value in comparison
  * @return      TRUE if 2 values are the same, FALSE otherwise
  */
-UBool U_CALLCONV dtitvinfHashTableValueComparator(UHashTok val1, UHashTok val2);
+static UBool U_CALLCONV hashTableValueComparator(UHashTok val1, UHashTok val2);
 
 U_CDECL_END
 
 UBool 
-U_CALLCONV dtitvinfHashTableValueComparator(UHashTok val1, UHashTok val2) {
+U_CALLCONV hashTableValueComparator(UHashTok val1, UHashTok val2) {
     const UnicodeString* pattern1 = (UnicodeString*)val1.pointer;
     const UnicodeString* pattern2 = (UnicodeString*)val2.pointer;
     UBool ret = TRUE;
@@ -603,7 +602,7 @@ DateIntervalInfo::initHash(UErrorCode& status) {
         status = U_MEMORY_ALLOCATION_ERROR;
         return NULL;
     }
-    hTable->setValueCompartor(dtitvinfHashTableValueComparator);
+    hTable->setValueCompartor(hashTableValueComparator);
     return hTable;
 }
 
