@@ -1,7 +1,7 @@
 /*
 ******************************************************************************
 *
-*   Copyright (C) 1998-2008, International Business Machines
+*   Copyright (C) 1998-2009, International Business Machines
 *   Corporation and others.  All Rights Reserved.
 *
 ******************************************************************************
@@ -205,11 +205,10 @@ you can use one of the following u_fprintf examples to display a UnicodeString.<
 /** Forward declaration of a Unicode-aware file @stable 3.0 */
 typedef struct UFILE UFILE;
 
-#ifndef U_HIDE_DRAFT_API
 /**
  * Enum for which direction of stream a transliterator applies to.
  * @see u_fsettransliterator
- * @draft 3.0
+ * @stable ICU 3.0
  */
 typedef enum { 
    U_READ = 1,
@@ -217,8 +216,6 @@ typedef enum {
    U_READWRITE =3  /* == (U_READ | U_WRITE) */ 
 } UFileDirection;
 
-#endif /* U_HIDE_DRAFT_API */
-
 /**
  * Open a UFILE.
  * A UFILE is a wrapper around a FILE* that is locale and codepage aware.
@@ -234,7 +231,7 @@ typedef enum {
  * read from the file. If this paramter is NULL the system default codepage
  * will be used.
  * @return A new UFILE, or NULL if an error occurred.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UFILE* U_EXPORT2
 u_fopen(const char    *filename,
@@ -253,7 +250,7 @@ u_fopen(const char    *filename,
  * read using the default codepage for <TT>locale</TT>, unless <TT>locale</TT>
  * is NULL, in which case the system default codepage will be used.
  * @return A new UFILE, or NULL if an error occurred.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UFILE* U_EXPORT2
 u_finit(FILE        *f,
@@ -272,7 +269,7 @@ u_finit(FILE        *f,
  * and parse output. If this parameter is NULL, the default locale will 
  * be used.
  * @return A new UFILE, or NULL if an error occurred.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UFILE* U_EXPORT2
 u_fstropen(UChar      *stringBuf,
@@ -282,7 +279,7 @@ u_fstropen(UChar      *stringBuf,
 /**
  * Close a UFILE.
  * @param file The UFILE to close.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT void U_EXPORT2
 u_fclose(UFILE *file);
@@ -293,7 +290,7 @@ u_fclose(UFILE *file);
  * @return Returns TRUE after the first read operation that attempts to
  * read past the end of the file. It returns FALSE if the current position is
  * not end of file.
- * @draft 3.0
+ * @stable ICU 3.0
 */
 U_DRAFT UBool U_EXPORT2
 u_feof(UFILE  *f);
@@ -304,7 +301,7 @@ u_feof(UFILE  *f);
  * made in the output stream - for example if a different type of
  * output is desired.)  The underlying OS level file is also flushed.
  * @param file The UFILE to flush.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT void U_EXPORT2
 u_fflush(UFILE *file);
@@ -312,7 +309,7 @@ u_fflush(UFILE *file);
 /**
  * Rewind the file pointer to the beginning of the file.
  * @param file The UFILE to rewind.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT void
 u_frewind(UFILE *file);
@@ -321,7 +318,7 @@ u_frewind(UFILE *file);
  * Get the FILE* associated with a UFILE.
  * @param f The UFILE
  * @return A FILE*, owned by the UFILE.  The FILE <EM>must not</EM> be closed.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT FILE* U_EXPORT2
 u_fgetfile(UFILE *f);
@@ -334,7 +331,7 @@ u_fgetfile(UFILE *f);
  * or <TT>u_fopen</TT>.
  * @param file The UFILE to set.
  * @return The locale whose conventions are used to format and parse output.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT const char* U_EXPORT2
 u_fgetlocale(UFILE *file);
@@ -345,7 +342,7 @@ u_fgetlocale(UFILE *file);
  * and parse output.
  * @param file The UFILE to query.
  * @return NULL if successful, otherwise a negative number.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_fsetlocale(UFILE      *file,
@@ -360,7 +357,7 @@ u_fsetlocale(UFILE      *file,
  * @param file The UFILE to query.
  * @return The codepage in which data is written to and read from the UFILE,
  * or NULL if an error occurred.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT const char* U_EXPORT2
 u_fgetcodepage(UFILE *file);
@@ -378,7 +375,7 @@ u_fgetcodepage(UFILE *file);
  * @param file The UFILE to set.
  * @return 0 if successful, otherwise a negative number.
  * @see u_frewind
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_fsetcodepage(const char   *codepage,
@@ -389,7 +386,7 @@ u_fsetcodepage(const char   *codepage,
  * Returns an alias to the converter being used for this file.
  * @param f The UFILE to get the value from
  * @return alias to the converter
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UConverter* U_EXPORT2 u_fgetConverter(UFILE *f);
 
@@ -403,7 +400,7 @@ U_DRAFT UConverter* U_EXPORT2 u_fgetConverter(UFILE *f);
  * @param patternSpecification A pattern specifying how <TT>u_fprintf</TT> will
  * interpret the variable arguments received and format the data.
  * @return The number of Unicode characters written to <TT>f</TT>.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_fprintf(UFILE         *f,
@@ -420,7 +417,7 @@ u_fprintf(UFILE         *f,
  * @param ap The argument list to use.
  * @return The number of Unicode characters written to <TT>f</TT>.
  * @see u_fprintf
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vfprintf(UFILE        *f,
@@ -433,7 +430,7 @@ u_vfprintf(UFILE        *f,
  * @param patternSpecification A pattern specifying how <TT>u_fprintf</TT> will
  * interpret the variable arguments received and format the data.
  * @return The number of Unicode characters written to <TT>f</TT>.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_fprintf_u(UFILE       *f,
@@ -450,7 +447,7 @@ u_fprintf_u(UFILE       *f,
  * @param ap The argument list to use.
  * @return The number of Unicode characters written to <TT>f</TT>.
  * @see u_fprintf_u
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vfprintf_u(UFILE      *f,
@@ -465,7 +462,7 @@ u_vfprintf_u(UFILE      *f,
  * @param f The UFILE to which to write.
  * @return A non-negative number if successful, EOF otherwise.
  * @see u_file_write
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_fputs(const UChar *s,
@@ -476,7 +473,7 @@ u_fputs(const UChar *s,
  * @param uc The UChar to write.
  * @param f The UFILE to which to write.
  * @return The character written if successful, EOF otherwise.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UChar32 U_EXPORT2
 u_fputc(UChar32  uc,
@@ -491,7 +488,7 @@ u_fputc(UChar32  uc,
  * @param f The UFILE to which to write.
  * @return The number of Unicode characters written.
  * @see u_fputs
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_file_write(const UChar    *ustring, 
@@ -509,7 +506,7 @@ u_file_write(const UChar    *ustring,
  * interpret the variable arguments received and parse the data.
  * @return The number of items successfully converted and assigned, or EOF
  * if an error occurred.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_fscanf(UFILE      *f,
@@ -527,7 +524,7 @@ u_fscanf(UFILE      *f,
  * @return The number of items successfully converted and assigned, or EOF
  * if an error occurred.
  * @see u_fscanf
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vfscanf(UFILE         *f,
@@ -541,7 +538,7 @@ u_vfscanf(UFILE         *f,
  * interpret the variable arguments received and parse the data.
  * @return The number of items successfully converted and assigned, or EOF
  * if an error occurred.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_fscanf_u(UFILE        *f,
@@ -559,7 +556,7 @@ u_fscanf_u(UFILE        *f,
  * @return The number of items successfully converted and assigned, or EOF
  * if an error occurred.
  * @see u_fscanf_u
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vfscanf_u(UFILE       *f,
@@ -577,7 +574,7 @@ u_vfscanf_u(UFILE       *f,
  * stored successively in <TT>s</TT> until a newline or EOF is
  * reached. A null character (U+0000) will be appended to <TT>s</TT>.
  * @return A pointer to <TT>s</TT>, or NULL if no characters were available.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UChar* U_EXPORT2
 u_fgets(UChar  *s,
@@ -591,7 +588,7 @@ u_fgets(UChar  *s,
  *
  * @param f The UFILE from which to read.
  * @return The UChar value read, or U+FFFF if no character was available.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UChar U_EXPORT2
 u_fgetc(UFILE   *f);
@@ -604,7 +601,7 @@ u_fgetc(UFILE   *f);
  * available, or U+FFFFFFFF if an ill-formed character was
  * encountered.
  * @see u_unescape()
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UChar32 U_EXPORT2
 u_fgetcx(UFILE  *f);
@@ -618,7 +615,7 @@ u_fgetcx(UFILE  *f);
  * @param c The UChar to put back on the stream.
  * @param f The UFILE to receive <TT>c</TT>.
  * @return The UChar32 value put back if successful, U_EOF otherwise.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UChar32 U_EXPORT2
 u_fungetc(UChar32   c,
@@ -632,7 +629,7 @@ u_fungetc(UChar32   c,
  * @param count The number of Unicode characters to read.
  * @param f The UFILE from which to read.
  * @return The number of Unicode characters read.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_file_read(UChar        *chars, 
@@ -656,7 +653,7 @@ u_file_read(UChar        *chars,
  * caller. If U_READWRITE is specified, only the WRITE transliterator
  * is returned. In most cases, the caller should call utrans_close()
  * on the result of this function.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT UTransliterator* U_EXPORT2
 u_fsettransliterator(UFILE *file, UFileDirection direction,
@@ -677,7 +674,7 @@ u_fsettransliterator(UFILE *file, UFileDirection direction,
  * interpret the variable arguments received and format the data.
  * @return The number of Unicode code units written to <TT>buffer</TT>. This
  * does not include the terminating null character.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_sprintf(UChar       *buffer,
@@ -699,7 +696,7 @@ u_sprintf(UChar       *buffer,
  * @return The number of Unicode characters that would have been written to
  * <TT>buffer</TT> had count been sufficiently large. This does not include
  * the terminating null character.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_snprintf(UChar      *buffer,
@@ -718,7 +715,7 @@ u_snprintf(UChar      *buffer,
  * @param ap The argument list to use.
  * @return The number of Unicode characters written to <TT>buffer</TT>.
  * @see u_sprintf
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vsprintf(UChar      *buffer,
@@ -743,7 +740,7 @@ u_vsprintf(UChar      *buffer,
  * @return The number of Unicode characters that would have been written to
  * <TT>buffer</TT> had count been sufficiently large.
  * @see u_sprintf
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vsnprintf(UChar     *buffer,
@@ -758,7 +755,7 @@ u_vsnprintf(UChar     *buffer,
  * @param patternSpecification A pattern specifying how <TT>u_sprintf</TT> will
  * interpret the variable arguments received and format the data.
  * @return The number of Unicode characters written to <TT>buffer</TT>.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_sprintf_u(UChar      *buffer,
@@ -779,7 +776,7 @@ u_sprintf_u(UChar      *buffer,
  * interpret the variable arguments received and format the data.
  * @return The number of Unicode characters that would have been written to
  * <TT>buffer</TT> had count been sufficiently large.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_snprintf_u(UChar     *buffer,
@@ -798,7 +795,7 @@ u_snprintf_u(UChar     *buffer,
  * @param ap The argument list to use.
  * @return The number of Unicode characters written to <TT>f</TT>.
  * @see u_sprintf_u
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vsprintf_u(UChar     *buffer,
@@ -823,7 +820,7 @@ u_vsprintf_u(UChar     *buffer,
  * @return The number of Unicode characters that would have been written to
  * <TT>f</TT> had count been sufficiently large.
  * @see u_sprintf_u
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vsnprintf_u(UChar *buffer,
@@ -841,7 +838,7 @@ u_vsnprintf_u(UChar *buffer,
  * interpret the variable arguments received and parse the data.
  * @return The number of items successfully converted and assigned, or EOF
  * if an error occurred.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_sscanf(const UChar   *buffer,
@@ -860,7 +857,7 @@ u_sscanf(const UChar   *buffer,
  * @return The number of items successfully converted and assigned, or EOF
  * if an error occurred.
  * @see u_sscanf
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vsscanf(const UChar  *buffer,
@@ -875,7 +872,7 @@ u_vsscanf(const UChar  *buffer,
  * interpret the variable arguments received and parse the data.
  * @return The number of items successfully converted and assigned, or EOF
  * if an error occurred.
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_sscanf_u(const UChar  *buffer,
@@ -894,7 +891,7 @@ u_sscanf_u(const UChar  *buffer,
  * @return The number of items successfully converted and assigned, or EOF
  * if an error occurred.
  * @see u_sscanf_u
- * @draft 3.0
+ * @stable ICU 3.0
  */
 U_DRAFT int32_t U_EXPORT2
 u_vsscanf_u(const UChar *buffer,
